package javax.swing.text;

import java.awt.event.ActionEvent;
import javax.swing.AbstractAction;
import java.io.IOException;
import java.io.ObjectInputStream;
import javax.swing.Action;
import java.util.Map;
import java.util.ArrayList;
import java.text.ParseException;
import javax.swing.JFormattedTextField;
import java.text.AttributedCharacterIterator;
import java.util.BitSet;
import java.text.Format;

public class InternationalFormatter extends DefaultFormatter
{
    private static final Format.Field[] EMPTY_FIELD_ARRAY;
    private Format format;
    private Comparable max;
    private Comparable min;
    private transient BitSet literalMask;
    private transient AttributedCharacterIterator iterator;
    private transient boolean validMask;
    private transient String string;
    private transient boolean ignoreDocumentMutate;
    
    public InternationalFormatter() {
        this.setOverwriteMode(false);
    }
    
    public InternationalFormatter(Format format) {
        this();
        this.setFormat(format);
    }
    
    public void setFormat(Format format) {
        this.format = format;
    }
    
    public Format getFormat() {
        return this.format;
    }
    
    public void setMinimum(Comparable min) {
        if (this.getValueClass() == null && min != null) {
            this.setValueClass(min.getClass());
        }
        this.min = min;
    }
    
    public Comparable getMinimum() {
        return this.min;
    }
    
    public void setMaximum(Comparable max) {
        if (this.getValueClass() == null && max != null) {
            this.setValueClass(max.getClass());
        }
        this.max = max;
    }
    
    public Comparable getMaximum() {
        return this.max;
    }
    
    @Override
    public void install(JFormattedTextField formattedTextField) {
        super.install(formattedTextField);
        this.updateMaskIfNecessary();
        this.positionCursorAtInitialLocation();
    }
    
    @Override
    public String valueToString(Object obj) throws ParseException {
        if (obj == null) {
            return "";
        }
        Format format = this.getFormat();
        if (format == null) {
            return obj.toString();
        }
        return format.format(obj);
    }
    
    @Override
    public Object stringToValue(String s) throws ParseException {
        Object o = this.stringToValue(s, this.getFormat());
        if (o != null && this.getValueClass() != null && !this.getValueClass().isInstance(o)) {
            o = super.stringToValue(o.toString());
        }
        try {
            if (!this.isValidValue(o, true)) {
                throw new ParseException("Value not within min/max range", 0);
            }
        }
        catch (ClassCastException obj) {
            throw new ParseException("Class cast exception comparing values: " + obj, 0);
        }
        return o;
    }
    
    public Format.Field[] getFields(int n) {
        if (this.getAllowsInvalid()) {
            this.updateMask();
        }
        Map<AttributedCharacterIterator.Attribute, Object> attributes = this.getAttributes(n);
        if (attributes != null && attributes.size() > 0) {
            ArrayList<Object> list = new ArrayList();
            list.addAll(attributes.keySet());
            return (Format.Field[])list.toArray(InternationalFormatter.EMPTY_FIELD_ARRAY);
        }
        return InternationalFormatter.EMPTY_FIELD_ARRAY;
    }
    
    @Override
    public Object clone() throws CloneNotSupportedException {
        InternationalFormatter internationalFormatter = (InternationalFormatter)super.clone();
        internationalFormatter.literalMask = null;
        internationalFormatter.iterator = null;
        internationalFormatter.validMask = false;
        internationalFormatter.string = null;
        return internationalFormatter;
    }
    
    @Override
    protected Action[] getActions() {
        if (this.getSupportsIncrement()) {
            return new Action[] { new InternationalFormatter.IncrementAction("increment", 1), new InternationalFormatter.IncrementAction("decrement", -1) };
        }
        return null;
    }
    
    Object stringToValue(String source, Format format) throws ParseException {
        if (format == null) {
            return source;
        }
        return format.parseObject(source);
    }
    
    boolean isValidValue(Object o, boolean b) {
        Comparable<Object> minimum = this.getMinimum();
        try {
            if (minimum != null && minimum.compareTo(o) > 0) {
                return false;
            }
        }
        catch (ClassCastException ex) {
            if (b) {
                throw ex;
            }
            return false;
        }
        Comparable<Object> maximum = this.getMaximum();
        try {
            if (maximum != null && maximum.compareTo(o) < 0) {
                return false;
            }
        }
        catch (ClassCastException ex2) {
            if (b) {
                throw ex2;
            }
            return false;
        }
        return true;
    }
    
    Map<AttributedCharacterIterator.Attribute, Object> getAttributes(int index) {
        if (this.isValidMask()) {
            AttributedCharacterIterator iterator = this.getIterator();
            if (index >= 0 && index <= iterator.getEndIndex()) {
                iterator.setIndex(index);
                return iterator.getAttributes();
            }
        }
        return null;
    }
    
    int getAttributeStart(AttributedCharacterIterator.Attribute attribute) {
        if (this.isValidMask()) {
            AttributedCharacterIterator iterator = this.getIterator();
            iterator.first();
            while (iterator.current() != '\uffff') {
                if (iterator.getAttribute(attribute) != null) {
                    return iterator.getIndex();
                }
                iterator.next();
            }
        }
        return -1;
    }
    
    AttributedCharacterIterator getIterator() {
        return this.iterator;
    }
    
    void updateMaskIfNecessary() {
        if (!this.getAllowsInvalid() && this.getFormat() != null) {
            if (!this.isValidMask()) {
                this.updateMask();
            }
            else if (!this.getFormattedTextField().getText().equals(this.string)) {
                this.updateMask();
            }
        }
    }
    
    void updateMask() {
        if (this.getFormat() != null) {
            Document document = this.getFormattedTextField().getDocument();
            this.validMask = false;
            if (document != null) {
                try {
                    this.string = document.getText(0, document.getLength());
                }
                catch (BadLocationException ex) {
                    this.string = null;
                }
                if (this.string != null) {
                    try {
                        this.updateMask(this.getFormat().formatToCharacterIterator(this.stringToValue(this.string)));
                    }
                    catch (ParseException ex2) {}
                    catch (IllegalArgumentException ex3) {}
                    catch (NullPointerException ex4) {}
                }
            }
        }
    }
    
    int getLiteralCountTo(int n) {
        int n2 = 0;
        for (int i = 0; i < n; ++i) {
            if (this.isLiteral(i)) {
                ++n2;
            }
        }
        return n2;
    }
    
    boolean isLiteral(int bitIndex) {
        return this.isValidMask() && bitIndex < this.string.length() && this.literalMask.get(bitIndex);
    }
    
    char getLiteral(int index) {
        if (this.isValidMask() && this.string != null && index < this.string.length()) {
            return this.string.charAt(index);
        }
        return '\0';
    }
    
    @Override
    boolean isNavigatable(int n) {
        return !this.isLiteral(n);
    }
    
    @Override
    void updateValue(Object o) {
        super.updateValue(o);
        this.updateMaskIfNecessary();
    }
    
    @Override
    void replace(DocumentFilter.FilterBypass filterBypass, int n, int n2, String s, AttributeSet set) throws BadLocationException {
        if (this.ignoreDocumentMutate) {
            filterBypass.replace(n, n2, s, set);
            return;
        }
        super.replace(filterBypass, n, n2, s, set);
        return;
    }
    
    private int getNextNonliteralIndex(int n, int n2) {
        int length = this.getFormattedTextField().getDocument().getLength();
        while (n >= 0 && n < length) {
            if (this.isLiteral(n)) {
                n += n2;
            }
            else {
                return n;
            }
        }
        return (n2 == -1) ? 0 : length;
    }
    
    @Override
    boolean canReplace(DefaultFormatter.ReplaceHolder replaceHolder) {
        if (!this.getAllowsInvalid()) {
            String text = replaceHolder.text;
            int n = (text != null) ? text.length() : 0;
            JFormattedTextField formattedTextField = this.getFormattedTextField();
            if (n == 0 && replaceHolder.length == 1 && formattedTextField.getSelectionStart() != replaceHolder.offset) {
                replaceHolder.offset = this.getNextNonliteralIndex(replaceHolder.offset, -1);
            }
            else if (this.getOverwriteMode()) {
                int offset = replaceHolder.offset;
                boolean n2 = false;
                int i = 0;
                while (i < replaceHolder.length) {
                    while (this.isLiteral(offset)) {
                        ++offset;
                    }
                    if (offset >= this.string.length()) {
                        n2 = true;
                        break;
                    }
                    offset = ++offset;
                    ++i;
                }
                if (n2 || formattedTextField.getSelectedText() == null) {
                    replaceHolder.length = offset - replaceHolder.offset;
                }
            }
            else if (n > 0) {
                replaceHolder.offset = this.getNextNonliteralIndex(replaceHolder.offset, 1);
            }
            else {
                replaceHolder.offset = this.getNextNonliteralIndex(replaceHolder.offset, -1);
            }
            ((InternationalFormatter.ExtendedReplaceHolder)replaceHolder).endOffset = replaceHolder.offset;
            ((InternationalFormatter.ExtendedReplaceHolder)replaceHolder).endTextLength = ((replaceHolder.text != null) ? replaceHolder.text.length() : 0);
        }
        else {
            ((InternationalFormatter.ExtendedReplaceHolder)replaceHolder).endOffset = replaceHolder.offset;
            ((InternationalFormatter.ExtendedReplaceHolder)replaceHolder).endTextLength = ((replaceHolder.text != null) ? replaceHolder.text.length() : 0);
        }
        int canReplace = super.canReplace(replaceHolder);
        if (canReplace && !this.getAllowsInvalid()) {
            ((InternationalFormatter.ExtendedReplaceHolder)replaceHolder).resetFromValue(this);
        }
        return canReplace;
    }
    
    @Override
    boolean replace(DefaultFormatter.ReplaceHolder replaceHolder) throws BadLocationException {
        int n = -1;
        int n2 = 1;
        int literalCountTo = -1;
        if (replaceHolder.length > 0 && (replaceHolder.text == null || replaceHolder.text.length() == 0) && (this.getFormattedTextField().getSelectionStart() != replaceHolder.offset || replaceHolder.length > 1)) {
            n2 = -1;
        }
        if (!this.getAllowsInvalid()) {
            if ((replaceHolder.text == null || replaceHolder.text.length() == 0) && replaceHolder.length > 0) {
                n = this.getFormattedTextField().getSelectionStart();
            }
            else {
                n = replaceHolder.offset;
            }
            literalCountTo = this.getLiteralCountTo(n);
        }
        if (super.replace(replaceHolder)) {
            if (n != -1) {
                this.repositionCursor(literalCountTo, ((InternationalFormatter.ExtendedReplaceHolder)replaceHolder).endOffset + ((InternationalFormatter.ExtendedReplaceHolder)replaceHolder).endTextLength, n2);
            }
            else {
                int endOffset = ((InternationalFormatter.ExtendedReplaceHolder)replaceHolder).endOffset;
                if (n2 == 1) {
                    endOffset += ((InternationalFormatter.ExtendedReplaceHolder)replaceHolder).endTextLength;
                }
                super.repositionCursor(endOffset, n2);
            }
            return true;
        }
        else {
            return false;
        }
    }
    
    private void repositionCursor(int n, int n2, int n3) {
        if (this.getLiteralCountTo(n2) != n2) {
            n2 -= n;
            for (int i = 0; i < n2; ++i) {
                if (this.isLiteral(i)) {
                    ++n2;
                }
            }
        }
        super.repositionCursor(n2, 1);
    }
    
    char getBufferedChar(int index) {
        if (this.isValidMask() && this.string != null && index < this.string.length()) {
            return this.string.charAt(index);
        }
        return '\0';
    }
    
    boolean isValidMask() {
        return this.validMask;
    }
    
    boolean isLiteral(Map map) {
        return map == null || map.size() == 0;
    }
    
    private void updateMask(AttributedCharacterIterator iterator) {
        if (iterator != null) {
            this.validMask = true;
            this.iterator = iterator;
            if (this.literalMask == null) {
                this.literalMask = new BitSet();
            }
            else {
                for (int i = this.literalMask.length() - 1; i >= 0; --i) {
                    this.literalMask.clear(i);
                }
            }
            iterator.first();
            while (iterator.current() != '\uffff') {
                int literal = this.isLiteral(iterator.getAttributes());
                int j;
                for (j = iterator.getIndex(); j < iterator.getRunLimit(); ++j) {
                    if (!(literal == 0)) {
                        this.literalMask.set(j);
                    }
                    else {
                        this.literalMask.clear(j);
                    }
                }
                iterator.setIndex(j);
            }
        }
    }
    
    boolean canIncrement(Object o, int n) {
        return o != null;
    }
    
    void selectField(Object o, int n) {
        AttributedCharacterIterator iterator = this.getIterator();
        if (iterator != null && o instanceof AttributedCharacterIterator.Attribute) {
            AttributedCharacterIterator.Attribute attribute = (AttributedCharacterIterator.Attribute)o;
            iterator.first();
            while (iterator.current() != '\uffff') {
                while (iterator.getAttribute(attribute) == null && iterator.next() != '\uffff') {}
                if (iterator.current() == '\uffff') {
                    continue;
                }
                int runLimit = iterator.getRunLimit(attribute);
                if (--n <= 0) {
                    this.getFormattedTextField().select(iterator.getIndex(), runLimit);
                    break;
                }
                iterator.setIndex(runLimit);
                iterator.next();
            }
        }
    }
    
    Object getAdjustField(int n, Map map) {
        return null;
    }
    
    private int getFieldTypeCountTo(Object o, int n) {
        AttributedCharacterIterator iterator = this.getIterator();
        int n2 = 0;
        if (iterator != null && o instanceof AttributedCharacterIterator.Attribute) {
            AttributedCharacterIterator.Attribute attribute = (AttributedCharacterIterator.Attribute)o;
            iterator.first();
            while (iterator.getIndex() < n) {
                while (iterator.getAttribute(attribute) == null && iterator.next() != '\uffff') {}
                if (iterator.current() != '\uffff') {
                    iterator.setIndex(iterator.getRunLimit(attribute));
                    iterator.next();
                    ++n2;
                }
                else {
                    break;
                }
            }
        }
        return n2;
    }
    
    Object adjustValue(Object o, Map map, Object o2, int n) throws BadLocationException, ParseException {
        return null;
    }
    
    boolean getSupportsIncrement() {
        return false;
    }
    
    void resetValue(Object o) throws BadLocationException, ParseException {
        Document document = this.getFormattedTextField().getDocument();
        String valueToString = this.valueToString(o);
        try {
            this.ignoreDocumentMutate = true;
            document.remove(0, document.getLength());
            document.insertString(0, valueToString, null);
        }
        finally {
            this.ignoreDocumentMutate = false;
        }
        this.updateValue(o);
    }
    
    private void readObject(ObjectInputStream objectInputStream) throws IOException, ClassNotFoundException {
        objectInputStream.defaultReadObject();
        this.updateMaskIfNecessary();
    }
    
    @Override
    DefaultFormatter.ReplaceHolder getReplaceHolder(DocumentFilter.FilterBypass filterBypass, int n, int n2, String s, AttributeSet set) {
        if (this.replaceHolder == null) {
            this.replaceHolder = new InternationalFormatter.ExtendedReplaceHolder();
        }
        return super.getReplaceHolder(filterBypass, n, n2, s, set);
    }
    
    static JFormattedTextField access$000(InternationalFormatter internationalFormatter) {
        return internationalFormatter.getFormattedTextField();
    }
    
    static JFormattedTextField access$100(InternationalFormatter internationalFormatter) {
        return internationalFormatter.getFormattedTextField();
    }
    
    static JFormattedTextField access$200(InternationalFormatter internationalFormatter) {
        return internationalFormatter.getFormattedTextField();
    }
    
    static int access$300(InternationalFormatter internationalFormatter, Object o, int n) {
        return internationalFormatter.getFieldTypeCountTo(o, n);
    }
    
    static void access$400(InternationalFormatter internationalFormatter) {
        internationalFormatter.invalidEdit();
    }
    
    static {
        EMPTY_FIELD_ARRAY = new Format.Field[0];
    }
    
    static class ExtendedReplaceHolder extends DefaultFormatter.ReplaceHolder
    {
        int endOffset;
        int endTextLength;
        
        void resetFromValue(InternationalFormatter internationalFormatter) {
            this.offset = 0;
            try {
                this.text = internationalFormatter.valueToString(this.value);
            }
            catch (ParseException ex) {
                this.text = "";
            }
            this.length = this.fb.getDocument().getLength();
        }
    }
    
    private class IncrementAction extends AbstractAction
    {
        private int direction;
        
        IncrementAction(String name, int direction) {
            super(name);
            this.direction = direction;
        }
        
        @Override
        public void actionPerformed(ActionEvent actionEvent) {
            if (InternationalFormatter.access$000(InternationalFormatter.this).isEditable()) {
                if (InternationalFormatter.this.getAllowsInvalid()) {
                    InternationalFormatter.this.updateMask();
                }
                boolean n = false;
                if (InternationalFormatter.this.isValidMask()) {
                    int selectionStart = InternationalFormatter.access$100(InternationalFormatter.this).getSelectionStart();
                    if (selectionStart != -1) {
                        AttributedCharacterIterator iterator = InternationalFormatter.this.getIterator();
                        iterator.setIndex(selectionStart);
                        Map<AttributedCharacterIterator.Attribute, Object> attributes = iterator.getAttributes();
                        Object adjustField = InternationalFormatter.this.getAdjustField(selectionStart, attributes);
                        if (InternationalFormatter.this.canIncrement(adjustField, selectionStart)) {
                            try {
                                Object stringToValue = InternationalFormatter.this.stringToValue(InternationalFormatter.access$200(InternationalFormatter.this).getText());
                                int access$300 = InternationalFormatter.access$300(InternationalFormatter.this, adjustField, selectionStart);
                                Object adjustValue = InternationalFormatter.this.adjustValue(stringToValue, attributes, adjustField, this.direction);
                                if (adjustValue != null && InternationalFormatter.this.isValidValue(adjustValue, false)) {
                                    InternationalFormatter.this.resetValue(adjustValue);
                                    InternationalFormatter.this.updateMask();
                                    if (InternationalFormatter.this.isValidMask()) {
                                        InternationalFormatter.this.selectField(adjustField, access$300);
                                    }
                                    n = true;
                                }
                            }
                            catch (ParseException ex) {}
                            catch (BadLocationException ex2) {}
                        }
                    }
                }
                if (n == 0) {
                    InternationalFormatter.access$400(InternationalFormatter.this);
                }
            }
        }
    }
}
